//////////////////////////////////////////////////////////////////////////////
//   ///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "_DRAW_3D.h"
#include "_3D.h"

//////////////////////////////////////////////////////////////////////////////
//   /////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
int H = 210;
int P = 32;

void Falloff(int f)
{ 
  DrawRect(RegionFull(),RGBis(f,f,f),modeMUL);
};

//////////////////////////////////////////////////////////////////////////////
inline tfloat dist(tfloat x1, tfloat y1, tfloat x2, tfloat y2)
{
  x2 = x2 - x1; y2 = (y2 - y1) * 4;
  return sqrt(x2 * x2 + y2 * y2);
};

//////////////////////////////////////////////////////////////////////////////
inline dwRGB DIColor(dwRGB c, int m = 255)
{ 
  return RGBmultV(c, DrawIntensityI * m / 255);
};

//////////////////////////////////////////////////////////////////////////////
// () GArray //////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
class GArray
{ public:
  int xD, yD, yS, vFilter, Dots, Mode; 
  tfloat vLevel, vMin, vMax, vOver, vLimit, iK1, iK2, iDeep, sPaint, sFalloff;
  tfloat_arr *Value, *Speed;
  dwRGB Colors[1024];
  int Count;

  tfloat (*EPaintFunc)(tfloat);

  int gx(int x) { return (xD + x) % xD; };
  int gy(int y) { return (yD + y) % yD; };

  void ColorFill(dwRGB c1, dwRGB c2)
  {
    dwRGB c;
    for (int i = 0; i <= yD; i++) Colors[i] = RGBsubV(RGBblend(c1, c2, i * 255 / yD),(i & 0x01) * 15);
    //for (int i = 0; i < Count - 2; i += 2) { c = Colors[i]; Colors[i] = Colors[i + 1]; Colors[i] = c; };
  };

  void SetMinMax(tfloat mn, tfloat mx, tfloat vl = 0.25, tfloat vo = -5) {vMin = mn; vMax = mx, vLimit = vl; vOver = vo; };

  void Create()                             //  
  {
    Done();  
    xD = DrawXR / 8 + 1; yD = DrawYR  / P - 1;              //    -
    yS = DrawYR - yD * P;
    Value = (tfloat_arr *)malloc(xD * yD * sizeof(tfloat)); //  
    Speed = (tfloat_arr *)malloc(xD * yD * sizeof(tfloat));
    for (int y = 0; y < yD; y++)
    for (int x = 0; x < xD; x++)                            //     
    {(*Value)[y * xD + x] = vLevel;(*Speed)[y * xD + x] = 0;
    };
  };

  void Done() { if (Value) free(Value); if (Speed) free(Speed); Speed = Value = NULL; };

  GArray()                                   // 
  {
    Mode = modeADD; vFilter = 0; vFilter = 0; vLevel = 0; Dots = 1;
    SetMinMax(-3.75, 3.75, 0.15);  Value = Speed = NULL;
    iK1 = iK2 = 0.025; iDeep = 11; sPaint = 0.75; sFalloff = 0.97;
  };

  ~GArray() { Done(); };                     //  

  void Paint(int x, int y, int rad)          // 
  { 
    tfloat r;
    int xx = x / 8, yy = (y - yS) / P;
    for (int y = yy - rad; y <= yy + rad; y++)
    for (int x = xx - rad; x <= xx + rad; x++)
    if ((r = dist(xx, yy, x, y)) <= rad) floatLinc((*Speed)[gy(y) * xD + gx(x)], vMax,(1 - Hermit(r / rad)) * sPaint);
  };

  void Draw()                                // 
  {
    tfloat v, vD = vMax - vMin;
    int d, x, y, ln, lnx;
    for (y = 0; y < yD; y++)                                            //        : 
    { ln = y * xD;                                                      //  x-
      for (x = 0; x < xD; x++)
      { lnx = ln + x;(*Speed)[lnx] -= iK1 * ((v = (*Value)[lnx]) - vLevel); //   
        for (d = 1; d <= iDeep; d++)                                    //      - iDeep
       (*Speed)[lnx] += iK2 * ((*Value)[ln + gx(x - d)] + (*Value)[ln + gx(x + d)] - 2 * v) / d;
      };
    };
    for (x = 0; x < xD; x++)                                            //        : 
    { for (y = 0; y < yD; y++)
      { lnx = y * xD + x; (*Speed)[lnx] -= iK1 * ((v = (*Value)[lnx]) - vLevel); //   
        for (d = 1; d <= iDeep / 4; d++)                                    //      - iDeep
       (*Speed)[lnx] += iK2 * ((*Value)[xD * gy(y - d) + x] + (*Value)[xD * gy(y + d) + x] - 2 * v) / (d * 2);
      };
    };

    int xx, yy;
    DrawRect(RegionIs(0, 0, DrawXL, H), Colors[0]);
    for (int y = 0; y < yD; y++)
    {
      for (int x = 0; x < xD; x++)
      { lnx = y * xD + x;  v = (*Value)[lnx] += (*Speed)[lnx] *= sFalloff; v *= 7;
        if (v < - (H - P)) v = - (H - P);
        xx = x * 8; yy = yS + y * P;
        if (x == 0) DrawPolyMoveTo(xx, yy - v); else DrawPolyLineTo(xx, yy - v);   
      };
      DrawPolyLineTo(xx, yy + H); DrawPolyLineTo(0, yy + H);
      DrawPoly(Colors[y + 1]);
      if (Dots)
      for (int x = 0; x < xD; x++)
      { lnx = y * xD + x;  v = (*Value)[lnx] * 7;
        xx = x * 8; yy = yS + y * P;
        DrawCircle(xx, yy - v, 0.5 + fabs(v / 100), RGBis(255, 255, 255));
      };

    };
  };
};

//////////////////////////////////////////////////////////////////////////////
//    /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
GArray garray;

//////////////////////////////////////////////////////////////////////////////////////////////////
int MousePressNow = no;

void mouseLBDown()                           //  []  
{ 
  MousePressNow = Yes;
};

void mouseLBUp()                           //  []  
{ 
  MousePressNow = no;
};

//////////////////////////////////////////////////////////////////////////////////////////////////
int keyExit = 0, HelpTimer = 0, BrushSize = 17, ColorI = 0;

void KeysLook(int force = no)                   //   
{
  if (!force && !II.AnyKey) return;

  if (II.KeyUp[_Enter_]) ColorI = (ColorI + 1) % 3;  
  if (II.KeyUp[_Quote_]) ColorI = (3 + ColorI - 1) % 3;
  if (II.KeyUp[_Space_]) garray.Dots = 1 - garray.Dots;


  switch(ColorI)
  { case 0: garray.ColorFill(RGBis(0, 155, 255), RGBis(255, 155, 0)); break;
    case 1: garray.ColorFill(RGBis(255, 155, 0), RGBis(155, 0, 0)); break; 
    case 2: garray.ColorFill(RGBis(180, 200, 0), RGBis(255, 100, 0)); break; 
  };/**/


  if (II.KeyUp[_1_]) BrushSize = 7;
  if (II.KeyUp[_2_]) BrushSize = 11; 
  if (II.KeyUp[_3_]) BrushSize = 17; 
  if (II.KeyUp[_4_]) BrushSize = 25;

  if (II.KeyUp[_7_]) garray.sPaint = 0.25;
  if (II.KeyUp[_8_]) garray.sPaint = 0.75; 
  if (II.KeyUp[_9_]) garray.sPaint = 1.25; 
  if (II.KeyUp[_0_]) garray.sPaint = 2;

  if (II.KeyUp[_F1_]) if (HelpTimer) HelpTimer = 0; else HelpTimer = 25 * 10;
  keyExit = (II.KeyUp[_Esc_]);
  II.Reset();
};


//////////////////////////////////////////////////////////////////////////////
//     /////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
TextSPR *info;

void CommonsInit()
{ RandomSeed();

  FC_FpsSet(AUTO);            //    
  FC_DrawPaint = Yes;         //  fps   DrawPaint();
  
  Draw2D_Init();
  ForceActivate();
  ScreenLoadWait();

  info = new TextSPR(L"TYPE_EN_RU");

  garray.Create(); //garray.vRadDrawLim = 0.5;
  garray.ColorFill(RGBis(0, 155, 255), RGBis(255, 155, 0));

  OnMouseLButtonDownDo(mouseLBDown);   //     
  OnMouseLButtonUpDo(mouseLBUp);   //     
//  OnKeyUpDo(keyPressed);       //     
  KeysLook(Yes);                //     ...
};


//////////////////////////////////////////////////////////////////////////////////////////////////
void CommonsDone()
{
  delete info;
  Draw2D_Done();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
void DrawInfo()
{
  tRegion r = RegionCenterIs(208, 250);
  DrawRegionStore.Save(r);
  tfRegion fr = RegionTofRegion(r);
  //DrawRect(r, RGBis(50, 50, 50), modeMUL); 
  fDrawRoundRect(fr, 20, RGBis(50, 50, 50), modeMUL); 
  info->Place(0, 0, modeADD + fromCC, "Conrol Keys:\n\n\nQutes, Enter - change Color\n\nSpace - set On/Off Dots\n...............................................................................\n\n1, 2, 3, 4 - set Brush Size\n\n7, 8, 9, 0 - set Brush Force\n...............................................................................\n\nF1 - hide/show Help\n\nEsc - Exit\n...............................................................................\n\n\nAnd... - Mouse Click & Motion"); info->Draw();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
void DrawAll()
{
  if (MousePressNow) garray.Paint(II.Mouse.X, II.Mouse.Y, BrushSize);
  garray.Draw();               // 
  if (HelpTimer) DrawInfo();
  DrawPaint();
  //if (HelpTimer) { DrawRegionStore.Restore();  HelpTimer--; };
};

//////////////////////////////////////////////////////////////////////////////
//   //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void Main()
{
  CommonsInit();                  //   ;  

  SetDrawIntensity(0.5);          //      
  //SetCursor(LoadCursor(NULL,IDC_CROSS));
  //SetCursor(NULL);

  while (!keyExit)
   { Falloff(120);
     KeysLook();
     DrawAll();
   };   

  CommonsDone();                  //   ;
};
//////////////////////////////////////////////////////////////////////////////
int LetProcess()
{
  WinName = L"CG : Array Wave"; //    
  VS_CColor = RGBis(0,0,0);    //   
  DrawSet(FullScreen);         //   ( )  , FullScreen   
  //SetDraw(640,480);
  ProcessFunc = Main;           //    -  
  return on;
};
int Init_LetProcess = LetProcess();

